@@ -1,6 +1,6 @@
 /* Routines for handling mode flags and strings.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -39,6 +39,7 @@ ModeData chanusermodes[] = {
 /* The following are initialized by mode_setup(): */
 int32 usermode_reg;		/* Usermodes applied to registered nicks */
 int32 chanmode_reg;		/* Chanmodes applied to registered chans */
+int32 chanmode_regonly;		/* Chanmodes indicating regnick-only channels*/
 int32 chanmode_opersonly;	/* Chanmodes indicating oper-only channels */
 int32 chanusermode_owner;	/* Chanuser-modes applied to channel owner */
 char  chanmode_multiple[257];	/* Chanmodes that can be set multiple times */
@@ -92,6 +93,8 @@ void mode_setup(void)
 	    uint32 tmp = (uint32) modelist[i].flag;
 	    if (modelist[i].info & MI_REGISTERED)
 		chanmode_reg |= tmp;
+	    if (modelist[i].info & MI_REGNICKS_ONLY)
+		chanmode_regonly |= tmp;
 	    if (modelist[i].info & MI_OPERS_ONLY)
 		chanmode_opersonly |= tmp;
 	    if (modelist[i].info & MI_MULTIPLE)
