@@ -1,6 +1,6 @@
 /* Mode flag definitions.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -32,11 +32,11 @@ typedef struct {
     uint32 info;	/* What kind of mode this is (MI_* below) */
 } ModeData;
 
-#define MI_REGISTERED	0x01	/* Applied to all registered nicks/chans */
-#define MI_OPERS_ONLY	0x02	/* A channel that only opers may join */
-#define MI_MULTIPLE	0x04	/* Mode can be set multiple times (+b etc);
-				 *    only needed for channel modes */
-#define MI_CHANOWNER	0x08	/* Set for a channel owner (founder) */
+#define MI_REGISTERED	0x01	/* [UC] Applied to all regged nicks/chans */
+#define MI_OPERS_ONLY	0x02	/* [ C] Only opers may join */
+#define MI_MULTIPLE	0x04	/* [ C] Can be set multiple times (+b etc) */
+#define MI_CHANOWNER	0x08	/* [U ] Set for a channel owner (founder) */
+#define MI_REGNICKS_ONLY 0x10	/* [ C] Only registered/ID'd nicks may join */
 
 /* These bits are available for private use by protocol modules: (see
  * Unreal module for an example of usage) */
@@ -59,6 +59,7 @@ extern ModeData usermodes[256], chanmodes[256], chanusermodes[256];
 /* The following are initialized by mode_setup(): */
 extern int32 usermode_reg;	/* Usermodes applied to registered nicks */
 extern int32 chanmode_reg;	/* Chanmodes applied to registered chans */
+extern int32 chanmode_regonly;	/* Chanmodes indicating regnick-only channels*/
 extern int32 chanmode_opersonly;/* Chanmodes indicating oper-only channels */
 extern int32 chanusermode_owner;/* Chanuser-modes applied to channel owner */
 extern char chanmode_multiple[];/* Chanmodes that can be set multiple times */
