@@ -1,6 +1,6 @@
 /* Base routines for ChanServ access level handling.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -24,7 +24,6 @@ static Module *module;
 /*************************************************************************/
 
 /* Array of all access levels: */
-EXPORT_ARRAY(struct levelinfo_, levelinfo)
 LevelInfo levelinfo[] = {
     /* Dummy entry for channel owner; flags filled in by init_access */
     { CA_AUTOOWNER, ACCLEV_FOUNDER, "",        -1,
@@ -41,7 +40,7 @@ LevelInfo levelinfo[] = {
     /* Internal use; not settable by users.  These two levels change based
      * on the SECUREOPS and RESTRICTED settings. */
     { CA_AUTODEOP,              -1, "",            -1,
-	  CL_CLEAR_MODE|CL_LESSEQUAL, { cumode: {"oh",0} } },
+	  CL_CLEAR_MODE|CL_LESSEQUAL, { cumode: {"oha",0} } },
     { CA_NOJOIN,              -100, "",            -1,
 	  CL_OTHER|CL_LESSEQUAL },
 
@@ -240,6 +239,16 @@ int check_access_cmd(User *user, ChannelInfo *ci, const char *command,
     return -1;
 }
 
+/*************************************************************************/
+
+/* Return the levelinfo[] array.  Used by the httpd/dbaccess module. */
+
+EXPORT_FUNC(get_levelinfo)
+LevelInfo *get_levelinfo(void)
+{
+    return levelinfo;
+}
+
 /*************************************************************************/
 /************************** Local-use routines ***************************/
 /*************************************************************************/
