@@ -1,6 +1,6 @@
 /* ChanServ-related structures.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -97,8 +97,8 @@ struct channelinfo_ {
     char *mlock_link;		/* +L (Unreal, trircd) */
     char *mlock_flood;		/* +f (Unreal, etc.) */
     int32 mlock_joindelay;	/* +J (trircd) */
-    int32 mlock_joinrate1;	/* +j (Bahamut) */
-    int32 mlock_joinrate2;	/* +j (Bahamut) */
+    int32 mlock_joinrate1;	/* +j (Bahamut/Unreal) */
+    int32 mlock_joinrate2;	/* +j (Bahamut/Unreal) */
 
     char *entry_message;	/* Notice sent on entering channel */
 
@@ -143,6 +143,9 @@ struct channelinfo_ {
 /* Hide mode lock from INFO */
 #define CI_HIDE_MLOCK	0x00008000
 
+/* All channel flags */
+#define CI_ALLFLAGS	0x0000FEFF
+
 /*************************************************************************/
 
 /* Indices for ci->levels[]: (DO NOT REORDER THESE unless you hack
@@ -209,6 +212,7 @@ E int check_access_cmd(User *user, ChannelInfo *ci, const char *command,
 		       const char *subcommand);
 E int check_channel_limit(NickGroupInfo *ngi, int *max_ret);
 E void reset_levels(ChannelInfo *ci, int set);
+E struct levelinfo_ *get_levelinfo(void);
 
 /*************************************************************************/
 
