@@ -1,6 +1,6 @@
 /* Routines to check validity of JOINs and mode changes.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -26,7 +26,7 @@ static int cb_check_chan_user_modes = -1;
 static int cb_check_kick = -1;
 
 static void local_set_cumodes(Channel *c, char plusminus, int32 modes,
-			      const char *nick);
+			      struct c_userlist *cu);
 
 /*************************************************************************/
 /*************************************************************************/
@@ -173,7 +173,7 @@ void check_chan_user_modes(const char *source, struct c_userlist *u,
 	    if (!((oldmodes & cumode_h)
 		  && !((oldmodes^modes) & ~(CUMODE_v|cumode_h)))
 	    ) {
-		local_set_cumodes(c, '-', (modes & ~oldmodes), user->nick);
+		local_set_cumodes(c, '-', (modes & ~oldmodes), u);
 	    }
 	}
 	return;
@@ -218,7 +218,7 @@ void check_chan_user_modes(const char *source, struct c_userlist *u,
     if ((res & ~modes)
      && (oldmodes < 0 || is_servermode || (ci->flags & CI_ENFORCE))
     ) {
-	local_set_cumodes(c, '+', res & ~modes, user->nick);
+	local_set_cumodes(c, '+', res & ~modes, u);
 	if ((res & ~modes) & CUMODE_o) {
 	    ci->last_used = time(NULL);
 	    put_channelinfo(ci);
@@ -231,7 +231,7 @@ void check_chan_user_modes(const char *source, struct c_userlist *u,
 
     /* Check for mode subtractions */
     if (res & modes)
-	local_set_cumodes(c, '-', res & modes, user->nick);
+	local_set_cumodes(c, '-', res & modes, u);
 }
 
 /*************************************************************************/
@@ -243,7 +243,7 @@ struct csinhabitdata_ {
     char chan[CHANMAX];
     Timeout *to;
 };
-CSInhabitData *inhabit_list = NULL;
+static CSInhabitData *inhabit_list = NULL;
 
 
 /* Tiny helper routine to get ChanServ out of a channel after it went in. */
@@ -285,7 +285,7 @@ int check_kick(User *user, const char *chan)
     if (is_services_admin(user))
 	return 0;
 
-    i = call_callback_5(module, cb_check_kick, user, c, ci, &mask, &reason);
+    i = call_callback_5(module, cb_check_kick, user, chan, ci, &mask, &reason);
     if (i == 2)
 	return 0;
     else if (i == 1)
@@ -327,6 +327,15 @@ int check_kick(User *user, const char *chan)
 	goto kick;
     }
 
+    if ((ci->mlock_on & chanmode_regonly) && !user_identified(user)) {
+	/* User must have usermode_reg flags, i.e. be using a registered
+	 * nick and have identified, in order to join a chanmode_regonly
+	 * channel */
+	mask = create_mask(user, 1);
+	reason = getstring(user->ngi, CHAN_NOT_ALLOWED_TO_JOIN);
+	goto kick;
+    }
+
     if (user_recognized(user))
 	ngi = user->ngi;
     else
@@ -437,11 +446,11 @@ int check_topiclock(Channel *c, time_t topic_time)
 /*************************************************************************/
 
 /* Helper routine for check_chan_user_modes(): sets all of the given modes
- * on client `nick' in channel `c'.
+ * on client `cu' in channel `c'.
  */
 
 static void local_set_cumodes(Channel *c, char plusminus, int32 modes,
-			      const char *nick)
+			      struct c_userlist *cu)
 {
     char buf[3], modestr[BUFSIZE], *s;
 
@@ -452,8 +461,14 @@ static void local_set_cumodes(Channel *c, char plusminus, int32 modes,
     s = modestr;
     while (*s) {
 	buf[1] = *s++;
-	set_cmode(s_ChanServ, c, buf, nick);
+	set_cmode(s_ChanServ, c, buf, cu->user->nick);
     }
+    /* Set user's modes now, so check_chan_user_modes() can properly
+     * determine whether subsequent modes should be set or not */
+    if (plusminus == '+')
+	cu->mode |= modes;
+    else if (plusminus == '-')
+	cu->mode &= ~modes;
 }
 
 /*************************************************************************/
