@@ -1,6 +1,6 @@
 /* Main ChanServ module.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -261,7 +261,7 @@ static int chanserv_whois(const char *source, char *who, char *extra)
 	     ServiceUser, ServiceHost, desc_ChanServ);
     send_cmd(ServerName, "312 %s %s %s :%s", source, who,
 	     ServerName, ServerDesc);
-    send_cmd(ServerName, "318 End of /WHOIS response.");
+    send_cmd(ServerName, "318 %s %s End of /WHOIS response.", source, who);
     return 1;
 }
 
@@ -584,7 +584,7 @@ static void do_help(User *u)
 	    notice_help(s_ChanServ, u, CHAN_OPER_HELP_COMMANDS_END);
 	}
     } else if (!CSEnableRegister && is_oper(u) && stricmp(cmd,"REGISTER")==0) {
-	notice_help(s_ChanServ, u, CHAN_HELP_REGISTER);
+	notice_help(s_ChanServ, u, CHAN_HELP_REGISTER, s_NickServ);
 	notice_help(s_ChanServ, u, CHAN_HELP_REGISTER_ADMINONLY);
     } else if (stricmp(cmd, "LIST") == 0) {
 	if (is_oper(u))
@@ -622,7 +622,6 @@ static void do_help(User *u)
 		|| stricmp(cmd, "DEPROTECT") == 0
 		|| stricmp(cmd, "INVITE") == 0
 		|| stricmp(cmd, "UNBAN") == 0
-		|| stricmp(cmd, "KICK") == 0
 		|| stricmp(cmd, "TOPIC") == 0
 		|| stricmp(cmd, "CLEAR") == 0
 		|| stricmp(cmd, "STATUS") == 0)
@@ -1048,7 +1047,7 @@ static void do_list(User *u)
 			if (ci->flags & CI_NOEXPIRE)
 			    noexpire_char = '!';
 			if (ci->suspendinfo)
-			    noexpire_char = '*';
+			    suspended_char = '*';
 		    }
 
 		    /* This can only be true for SADMINS - normal users
@@ -1357,6 +1356,11 @@ static void do_cskick(User *u)
 		return;
 	    }
 	}
+	/* Also prevent Services opers and above from being kicked */
+	if (is_services_oper(target_user)) {
+	    notice_lang(s_ChanServ, u, CHAN_KICK_PROTECTED, target, chan);
+	    return;
+	}
 
 	/* Construct reason string: "KICK by Nick" / "KICK by Nick (reason)" */
 	if (reason && !*reason)
