@@ -1,6 +1,6 @@
 /* SET command handling.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -23,6 +23,7 @@ static Module *module;
 
 static int cb_set = -1;
 static int cb_set_mlock = -1;
+static int cb_unset = -1;
 
 /*************************************************************************/
 
@@ -66,8 +67,8 @@ void do_set(User *u)
     }
 
     if (cmd) {
-	if (stricmp(cmd, "PASSWORD") == 0 || stricmp(cmd, "DESC") == 0
-	 || stricmp(cmd, "TOPIC") == 0 || stricmp(cmd, "ENTRYMSG") == 0)
+	if (stricmp(cmd, "DESC") == 0 || stricmp(cmd, "TOPIC") == 0
+         || stricmp(cmd, "ENTRYMSG") == 0)
 	    param = strtok_remaining();
 	else
 	    param = strtok(NULL, " ");
@@ -152,6 +153,8 @@ void do_unset(User *u)
 	notice_lang(s_ChanServ, u, CHAN_X_FORBIDDEN, chan);
     } else if (!is_servadmin && !check_access_cmd(u, ci, "SET", cmd)) {
 	notice_lang(s_ChanServ, u, ACCESS_DENIED);
+    } else if (call_callback_3(module, cb_unset, u, ci, cmd) > 0) {
+	return;
     } else if (stricmp(cmd, "SUCCESSOR") == 0) {
 	if (!is_servadmin && !is_founder(u, ci)) {
 	    notice_lang(s_ChanServ, u, CHAN_IDENTIFY_REQUIRED,s_ChanServ,chan);
@@ -257,6 +260,7 @@ static void do_set_successor(User *u, ChannelInfo *ci, char *param)
 static void do_set_password(User *u, ChannelInfo *ci, char *param)
 {
     int len = strlen(param), max;
+    User *u2;
 
     /* Password length check and truncation (like NickServ REGISTER) */
     max = encrypt_check_len(len, PASSMAX);
@@ -286,6 +290,19 @@ static void do_set_password(User *u, ChannelInfo *ci, char *param)
 				"channel \2%s\2", u->nick, ci->name);
     }
     put_channelinfo(ci);
+    /* Clear founder privileges from all other users who might have
+     * identified earlier. */
+    for (u2 = first_user(); u2; u2 = next_user()) {
+        if (u2 != u) {
+            struct u_chaninfolist *c, *c2;
+            LIST_FOREACH_SAFE (c, u2->id_chans, c2) {
+                if (irc_stricmp(c->chan, ci->name) == 0) {
+                    LIST_REMOVE(c, u2->id_chans);
+                    free(c);
+                }
+            }
+        }
+    }
 }
 
 /*************************************************************************/
@@ -570,7 +587,8 @@ int init_set(Module *my_module)
     module = my_module;
     cb_set = register_callback(module, "SET");
     cb_set_mlock = register_callback(module, "SET MLOCK");
-    if (cb_set < 0 || cb_set_mlock < 0) {
+    cb_unset = register_callback(module, "UNSET");
+    if (cb_set < 0 || cb_set_mlock < 0 || cb_unset < 0) {
 	module_log("set: Unable to register callbacks");
 	exit_set();
 	return 0;
@@ -582,6 +600,7 @@ int init_set(Module *my_module)
 
 void exit_set()
 {
+    unregister_callback(module, cb_unset);
     unregister_callback(module, cb_set_mlock);
     unregister_callback(module, cb_set);
 }
