@@ -1,6 +1,6 @@
 /* Database file handling routines.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -245,7 +245,7 @@ int close_db(dbFILE *f)
 
 /*************************************************************************/
 
-int read_int8(uint8 *ret, dbFILE *f)
+int read_int8(int8 *ret, dbFILE *f)
 {
     int c = fgetc(f->fp);
     if (c == EOF)
@@ -254,7 +254,13 @@ int read_int8(uint8 *ret, dbFILE *f)
     return 0;
 }
 
-int write_int8(uint8 val, dbFILE *f)
+/* Alternative version of read_int8() to avoid GCC's pointer signedness
+ * warnings when reading into an unsigned variable: */
+int read_uint8(uint8 *ret, dbFILE *f) {
+    return read_int8((int8 *)ret, f);
+}
+
+int write_int8(int8 val, dbFILE *f)
 {
     if (fputc(val, f->fp) == EOF)
 	return -1;
@@ -263,9 +269,9 @@ int write_int8(uint8 val, dbFILE *f)
 
 /*************************************************************************/
 
-/* These two are inline to help out {read,write}_string. */
+/* These are inline to help out {read,write}_string. */
 
-inline int read_int16(uint16 *ret, dbFILE *f)
+inline int read_int16(int16 *ret, dbFILE *f)
 {
     int c1, c2;
 
@@ -277,7 +283,11 @@ inline int read_int16(uint16 *ret, dbFILE *f)
     return 0;
 }
 
-inline int write_int16(uint16 val, dbFILE *f)
+inline int read_uint16(uint16 *ret, dbFILE *f) {
+    return read_int16((int16 *)ret, f);
+}
+
+inline int write_int16(int16 val, dbFILE *f)
 {
     fputc((val>>8), f->fp);
     if (fputc(val, f->fp) == EOF)
@@ -287,7 +297,7 @@ inline int write_int16(uint16 val, dbFILE *f)
 
 /*************************************************************************/
 
-int read_int32(uint32 *ret, dbFILE *f)
+int read_int32(int32 *ret, dbFILE *f)
 {
     int c1, c2, c3, c4;
 
@@ -301,7 +311,11 @@ int read_int32(uint32 *ret, dbFILE *f)
     return 0;
 }
 
-int write_int32(uint32 val, dbFILE *f)
+int read_uint32(uint32 *ret, dbFILE *f) {
+    return read_int32((int32 *)ret, f);
+}
+
+int write_int32(int32 val, dbFILE *f)
 {
     fputc((val>>24), f->fp);
     fputc((val>>16), f->fp);
@@ -365,7 +379,7 @@ int read_string(char **ret, dbFILE *f)
     char *s;
     uint16 len;
 
-    if (read_int16(&len, f) < 0)
+    if (read_uint16(&len, f) < 0)
 	return -1;
     if (len == 0) {
 	*ret = NULL;
