@@ -1,7 +1,7 @@
 /* Routines to load/save Services databases in the (hopefully one day
  * obsolete) format used by version 4.x.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -288,7 +288,7 @@ static NickInfo *load_one_nick(dbFILE *f, int32 ver)
 	    memos = smalloc(sizeof(Memo) * ngi->memos.memos_count);
 	    ngi->memos.memos = memos;
 	    ARRAY_FOREACH (i, ngi->memos.memos) {
-		SAFE(read_int32(&ngi->memos.memos[i].number, f));
+		SAFE(read_uint32(&ngi->memos.memos[i].number, f));
 		SAFE(read_int16(&ngi->memos.memos[i].flags, f));
 		SAFE(read_int32(&tmp32, f));
 		ngi->memos.memos[i].time = tmp32;
@@ -362,7 +362,7 @@ static int load_one_nick_ext(dbFILE *f, int32 ver)
     free(ni->last_realmask);  /* copied from last_usermask */
     SAFE(read_string(&ni->last_realmask, f));
     if (ver >= 19)
-	SAFE(read_int32(&ni->id_stamp, f));
+	SAFE(read_uint32(&ni->id_stamp, f));
     if (ni == &dummy_ni)
 	free(ni->last_realmask);
     return 0;
@@ -383,7 +383,7 @@ static int load_one_nickgroup_ext(dbFILE *f, int32 ver)
     NickGroupInfo dummy_ngi;  /* for nonexisting nick groups */
     int i;
 
-    SAFE(read_int32(&group, f));
+    SAFE(read_uint32(&group, f));
     if (debug >= 2)
 	module_log("debug: loading nickgroup extension %u", group);
     if (!group) {
@@ -1013,7 +1013,7 @@ static ChannelInfo *load_one_channel(dbFILE *f, int32 ver)
 	memos = smalloc(sizeof(Memo) * ci->memos.memos_count);
 	ci->memos.memos = memos;
 	ARRAY_FOREACH (i, ci->memos.memos) {
-	    SAFE(read_int32(&ci->memos.memos[i].number, f));
+	    SAFE(read_uint32(&ci->memos.memos[i].number, f));
 	    SAFE(read_int16(&ci->memos.memos[i].flags, f));
 	    SAFE(read_int32(&tmp32, f));
 	    ci->memos.memos[i].time = tmp32;
@@ -1415,10 +1415,10 @@ int close_channel_db(const char *dbname)
 
 /* Local copies of OperServ data. */
 
-static int maxusercnt;
-static time_t maxusertime;
-static int8 no_supass;
-static char supass[PASSMAX];
+static int local_maxusercnt;
+static time_t local_maxusertime;
+static int8 local_no_supass;
+static char local_supass[PASSMAX];
 
 /*************************************************************************/
 
@@ -1426,16 +1426,16 @@ int get_operserv_data(int what, void *ret)
 {
     switch (what) {
       case OSDATA_MAXUSERCNT:
-	*((int *)ret) = maxusercnt;
+	*((int *)ret) = local_maxusercnt;
 	return 1;
       case OSDATA_MAXUSERTIME:
-	*((time_t *)ret) = maxusertime;
+	*((time_t *)ret) = local_maxusertime;
 	return 1;
       case OSDATA_SUPASS:
-	if (no_supass)
+	if (local_no_supass)
 	    *((char **)ret) = NULL;
 	else
-	    *((char **)ret) = supass;
+	    *((char **)ret) = local_supass;
 	return 1;
     }
     return 0;
@@ -1447,18 +1447,18 @@ int put_operserv_data(int what, void *ptr)
 {
     switch (what) {
       case OSDATA_MAXUSERCNT:
-	maxusercnt = *((int *)ptr);
+	local_maxusercnt = *((int *)ptr);
 	return 1;
       case OSDATA_MAXUSERTIME:
-	maxusertime = *((time_t *)ptr);
+	local_maxusertime = *((time_t *)ptr);
 	return 1;
       case OSDATA_SUPASS:
 	if (ptr) {
-	    no_supass = 0;
-	    memcpy(supass, ptr, PASSMAX);
+	    local_no_supass = 0;
+	    memcpy(local_supass, ptr, PASSMAX);
 	} else {
-	    no_supass = 1;
-	    memset(supass, 0, PASSMAX);
+	    local_no_supass = 1;
+	    memset(local_supass, 0, PASSMAX);
 	}
 	return 1;
     }
@@ -1481,10 +1481,10 @@ int open_operserv_db(const char *dbname)
 
     services_admins = services_opers = NULL;
     services_admins_count = services_opers_count = 0;
-    maxusercnt = 0;
-    maxusertime = 0;
-    no_supass = 1;
-    memset(supass, 0, sizeof(supass));
+    local_maxusercnt = 0;
+    local_maxusertime = 0;
+    local_no_supass = 1;
+    memset(local_supass, 0, sizeof(local_supass));
 
     SAFE(read_int16(&n, f));
     for (i = 0; i < n; i++) {
@@ -1506,14 +1506,14 @@ int open_operserv_db(const char *dbname)
     }
     if (ver >= 7) {
 	int32 tmp32;
-	SAFE(read_int32(&maxusercnt, f));
+	SAFE(read_int32(&local_maxusercnt, f));
 	SAFE(read_int32(&tmp32, f));
-	maxusertime = tmp32;
+	local_maxusertime = tmp32;
     }
     if (ver >= 9) {
-	SAFE(read_int8(&no_supass, f));
-	if (!no_supass)
-	    SAFE(read_buffer(supass, f));
+	SAFE(read_int8(&local_no_supass, f));
+	if (!local_no_supass)
+	    SAFE(read_buffer(local_supass, f));
     }
 
     close_db(f);
@@ -1562,11 +1562,11 @@ int sync_operserv_db(const char *dbname)
     ARRAY_FOREACH (i, services_opers)
 	SAFE(write_string(services_opers[i], f));
 
-    SAFE(write_int32(maxusercnt, f));
-    SAFE(write_int32(maxusertime, f));
-    SAFE(write_int8(no_supass, f));
-    if (!no_supass)
-	SAFE(write_buffer(supass, f));
+    SAFE(write_int32(local_maxusercnt, f));
+    SAFE(write_int32(local_maxusertime, f));
+    SAFE(write_int8(local_no_supass, f));
+    if (!local_no_supass)
+	SAFE(write_buffer(local_supass, f));
 
     SAFE(close_db(f));
     return 0;
