@@ -1,6 +1,6 @@
 /* IP address authorization module for HTTP server.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -160,7 +160,7 @@ static int do_AllowDenyHost(const char *filename, int linenum, char *param,
 
     if (strcmp(param, "*") == 0) {
 	/* All-hosts wildcard -> equivalent to 0.0.0.0/0 */
-	ip = "\0\0\0\0";
+	ip = (const uint8 *)"\0\0\0\0";
 	mask = 0;
     } else if ((ip = pack_ip(param)) != NULL) {
 	/* IP address -> okay as is */
@@ -181,7 +181,7 @@ static int do_AllowDenyHost(const char *filename, int linenum, char *param,
 	    if (hp->h_addrtype == AF_INET) {
 		for (i = 0; hp->h_addr_list[i]; i++) {
 		    char ipbuf[16];
-		    ip = hp->h_addr_list[i];
+		    ip = (const uint8 *)hp->h_addr_list[i];
 		    snprintf(ipbuf, sizeof(ipbuf), "%u.%u.%u.%u",
 			     ip[0], ip[1], ip[2], ip[3]);
 		    if (strlen(ipbuf) > 15) {
