@@ -1,6 +1,6 @@
 /* Database access module for HTTP server.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -934,16 +934,15 @@ static int handle_chanserv(Client *c, int *close_ptr, char *path)
 		   chanhtml);
 
     } else if (mode == MODE_LEVELS) {
-	LevelInfo *levelinfo;  /* imported from ChanServ */
+	LevelInfo *(*get_levelinfo)(void), *levelinfo;  /* from ChanServ */
 
-	levelinfo = get_module_symbol(module_chanserv, "levelinfo");
+	get_levelinfo = get_module_symbol(module_chanserv, "get_levelinfo");
 	sockprintf(c->socket,
 		   "<html><head><title>Access levels for channel \"%s\""
 		   "</title></head><body><h1 align=center>Access levels"
 		   " for channel \"%s\"</h1>", chanhtml, chanhtml);
-	if (!levelinfo) {
-	    module_log("Unable to resolve symbol `levelinfo' in module"
-		       " `chanserv/main'");
+	if (!get_levelinfo || !(levelinfo = get_levelinfo())) {
+	    module_log("Unable to retrieve ChanServ level data");
 	    sockprintf(c->socket, "<p><font color=red><b>Error accessing"
 		       " level data!</b></font>");
 	} else {
