@@ -1,6 +1,6 @@
 /* Main HTTP server module.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -760,7 +760,8 @@ static int do_ListenTo(const char *filename, int linenum, char *param)
     char *s, *t;
     long port;
     uint8 *ip;
-    char ipbuf[15+6+1];  /* aaa.bbb.ccc.ddd:ppppp\0 */
+    char *ipstr;
+    char ipbuf[15+1];  /* aaa.bbb.ccc.ddd\0 */
     int recursing = 0, i;
     static struct listento_ *new_ListenTo;
     static int new_ListenTo_count;
@@ -819,16 +820,17 @@ static int do_ListenTo(const char *filename, int linenum, char *param)
 
     if (strcmp(param, "*") == 0) {
 	/* "*" -> all addresses (NULL string) */
-	ip = NULL;
+	ipstr = NULL;
     } else if ((ip = pack_ip(param)) != NULL) {
 	/* IP address -> normalize (no leading zeros, etc.) */
-	snprintf(ipbuf, sizeof(ipbuf), "%u.%u.%u.%u", ip[0],ip[1],ip[2],ip[3]);
+	snprintf(ipbuf, sizeof(ipbuf), "%u.%u.%u.%u",
+		 ip[0], ip[1], ip[2], ip[3]);
 	if (strlen(ipbuf) > 15) {
 	    config_error(filename, linenum, "BUG: strlen(ipbuf) > 15 [%s]",
 			 ipbuf);
 	    return 0;
 	}
-	ip = ipbuf;
+	ipstr = ipbuf;
     } else {
 	/* hostname -> check for double recursion, then look up and
 	 *             recursively add addresses */
@@ -844,7 +846,7 @@ static int do_ListenTo(const char *filename, int linenum, char *param)
 	if ((hp = gethostbyname(param)) != NULL) {
 	    if (hp->h_addrtype == AF_INET) {
 		for (i = 0; hp->h_addr_list[i]; i++) {
-		    ip = hp->h_addr_list[i];
+		    ip = (uint8 *)hp->h_addr_list[i];
 		    snprintf(ipbuf, sizeof(ipbuf), "%u.%u.%u.%u",
 			     ip[0], ip[1], ip[2], ip[3]);
 		    if (strlen(ipbuf) > 15) {
@@ -874,8 +876,8 @@ static int do_ListenTo(const char *filename, int linenum, char *param)
 
     i = new_ListenTo_count;
     ARRAY_EXTEND(new_ListenTo);
-    if (ip)
-	strcpy(new_ListenTo[i].ip, ip); /* safe: strlen(ip)<16 checked above */
+    if (ipstr)
+	strcpy(new_ListenTo[i].ip, ipstr);/*safe: strlen(ip)<16 checked above*/
     else
 	memset(new_ListenTo[i].ip, 0, sizeof(new_ListenTo[i].ip));
     new_ListenTo[i].port = port;
