@@ -1,6 +1,6 @@
 /* NickServ-related structures.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -93,8 +93,6 @@ struct nickgroupinfo_ {
 
     int16 language;	/* Language selected by nickname owner (LANG_*) */
     int16 timezone;	/* Offset from UTC, in minutes */
-    channame_t *channels; /* Array of names of channels currently registered */
-    int16 channels_count; /* Number of channels currently registered */
     int16 channelmax;	/* Maximum number of registered channels allowed */
 
     char **access;	/* Array of access masks */
@@ -103,12 +101,15 @@ struct nickgroupinfo_ {
     char **ajoin;	/* Array of autojoin channel */
     int16 ajoin_count;	/* # of entries */
 
-    MemoInfo memos;
     char **ignore;	/* Array of memo ignore entries */
     int16 ignore_count;	/* # of entries */
 
+    MemoInfo memos;
+
     /* Online-only info: */
 
+    channame_t *channels; /* Array of names of channels currently registered */
+    int16 channels_count; /* Number of channels currently registered */
     User **id_users;	/* Users who have identified for this group (array) */
     int id_users_count;
     time_t last_sendauth; /* Time of last SENDAUTH (mail-auth module) */
@@ -121,6 +122,7 @@ struct nickgroupinfo_ {
 /* Nickname status flags: */
 #define NS_VERBOTEN	0x0002      /* Nick may not be registered or used */
 #define NS_NOEXPIRE	0x0004      /* Nick never expires */
+#define NS_PERMANENT	0x0006      /* All permanent flags */
 /* The following flags are temporary: */
 #define NS_KILL_HELD	0x8000      /* Nick is being held after a kill */
 #define NS_GUESTED	0x4000	    /* SVSNICK has been sent but nick has not
@@ -153,6 +155,8 @@ struct nickgroupinfo_ {
 #define NF_MEMO_FWD	0x00001000  /* Forward memos to E-mail address */
 #define NF_MEMO_FWDCOPY	0x00002000  /* Save copy of forwarded memos */
 
+#define NF_ALLFLAGS	0x00003FFB  /* All flags */
+
 
 /* Nickgroup OperServ privilege levels: */
 #define NP_SERVOPER	0x1000
@@ -180,9 +184,11 @@ struct nickgroupinfo_ {
 #define NICKGROUPINFO_INVALID	((NickGroupInfo *)PTR_INVALID)
 
 /* Authentication code reasons: */
+#define NICKAUTH_MIN		NICKAUTH_REGISTER
 #define NICKAUTH_REGISTER	1	/* Newly registered */
 #define NICKAUTH_SET_EMAIL	2	/* E-mail address changed */
 #define NICKAUTH_SETAUTH	3	/* SETAUTH command used */
+#define NICKAUTH_MAX		NICKAUTH_SETAUTH
 
 /*************************************************************************/
 
