@@ -1,6 +1,6 @@
 /* Nickname linking module (4.x compatibility version).
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -215,7 +215,7 @@ static void do_unlink(User *u)
     char *nick = strtok(NULL, " ");
     char *pass = strtok_remaining();
     int msg = -1;  /* -1 = flag meaning "error, abort" */
-    char *msgparam[2];
+    char *msgparam[2] = {NULL, NULL};
     int i;
 
     if (readonly && !is_services_admin(u)) {
