@@ -1,6 +1,6 @@
 /* Routines to handle the NickServ SET command.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -21,6 +21,7 @@
 static Module *module;
 static int cb_set = -1;
 static int cb_set_email = -1;
+static int cb_unset = -1;
 
 /*************************************************************************/
 
@@ -63,7 +64,7 @@ void do_set(User *u)
     } else {
 	ni = u->ni;
     }
-    if (cmd && stricmp(cmd, "PASSWORD") != 0 && stricmp(cmd, "INFO") != 0) {
+    if (cmd && stricmp(cmd, "INFO") != 0) {
 	param = strtok(param, " ");
 	extra = strtok(NULL, " ");
     }
@@ -146,6 +147,8 @@ void do_unset(User *u)
 	notice_lang(s_NickServ, u, INTERNAL_ERROR);
     } else if (!is_servadmin && !user_identified(u)) {
 	notice_lang(s_NickServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
+    } else if (call_callback_4(module, cb_unset, u, ni, ngi, cmd) > 0) {
+	return;
     } else if (stricmp(cmd, "URL") == 0) {
 	do_set_url(u, ngi, NULL);
     } else if (stricmp(cmd, "EMAIL") == 0) {
@@ -571,7 +574,8 @@ int init_set(Module *my_module)
     module = my_module;
     cb_set = register_callback(module, "SET");
     cb_set_email = register_callback(module, "SET EMAIL");
-    if (cb_set < 0 || cb_set_email < 0) {
+    cb_unset = register_callback(module, "UNSET");
+    if (cb_set < 0 || cb_set_email < 0 || cb_unset < 0) {
 	module_log("set: Unable to register callbacks");
 	exit_set();
 	return 0;
@@ -583,6 +587,7 @@ int init_set(Module *my_module)
 
 void exit_set()
 {
+    unregister_callback(module, cb_unset);
     unregister_callback(module, cb_set_email);
     unregister_callback(module, cb_set);
 }
