@@ -1,6 +1,6 @@
 /* NickServ utility routines.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -396,6 +396,10 @@ int validate_user(User *u)
     ni->user = u;
 
     if ((ni->status & NS_VERBOTEN) || ngi->suspendinfo) {
+	if (usermode_reg) {
+	    send_cmd(s_NickServ, "SVSMODE %s :-%s", u->nick,
+		     mode_flags_to_string(usermode_reg, MODE_USER));
+	}
 	notice_lang(s_NickServ, u, NICK_MAY_NOT_BE_USED);
 	notice_lang(s_NickServ, u, DISCONNECT_IN_1_MINUTE);
 	add_ns_timeout(ni, TO_SEND_433, 40);
@@ -456,6 +460,13 @@ int validate_user(User *u)
 
     }  /* if (!ngi->authcode) */
 
+    /* From here on down, the user is known to be not identified.  Clear
+     * any "registered nick" mode from them. */
+    if (usermode_reg) {
+	send_cmd(s_NickServ, "SVSMODE %s :-%s", u->nick,
+		 mode_flags_to_string(usermode_reg, MODE_USER));
+    }
+
     is_recognized = (call_callback_1(module, cb_check_recognized, u) == 1);
 
     if (!(ngi->flags & NF_SECURE) && !ngi->authcode && is_recognized) {
