@@ -1,6 +1,6 @@
 /* S-line module.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -310,6 +310,8 @@ static void do_szline(User *u) {
 	do_sline(MD_SZLINE, u);
 }
 
+static int check_add_sline(User *u, uint8 type, char *mask,
+			   time_t *expiry_ptr);
 static void do_add_sline(User *u, uint8 type, MaskData *md);
 static void do_del_sline(User *u, uint8 type, MaskData *md);
 
@@ -333,7 +335,7 @@ static MaskDataCmdInfo sline_cmd_info = {
     OPER_SLINE_COUNT,
 
     NULL,		/* function to mangle masks for add/delete */
-    NULL,		/* function to check validity of mask on add */
+    check_add_sline,	/* function to check validity of mask on add */
     do_add_sline,	/* function to call on mask addition */
     do_del_sline,	/* function to call on mask removal */
     NULL,		/* function to call for unknown commands */
@@ -366,6 +368,28 @@ static void do_sline(uint8 type, User *u)
 
 /*************************************************************************/
 
+static int check_add_sline(User *u, uint8 type, char *mask, time_t *expiry_ptr)
+{
+    char *t;
+
+#ifdef CLEAN_COMPILE
+    expiry_ptr = expiry_ptr;
+#endif
+    /* Make sure mask is not too general. */
+    if (strchr(mask,'*') != NULL && mask[strspn(mask,"*?")] == 0
+     && ((t = strchr(mask,'?')) == NULL || strchr(t+1,'?') == NULL)
+    ) {
+	char cmdname[7];
+	snprintf(cmdname, sizeof(cmdname), "S%cLINE", (char)type);
+	notice_lang(s_OperServ, u, OPER_SLINE_MASK_TOO_GENERAL, cmdname);
+	return 0;
+    }
+
+    return 1;
+}
+
+/*************************************************************************/
+
 static void do_add_sline(User *u, uint8 type, MaskData *md)
 {
     if (WallOSSline) {
