@@ -1,6 +1,6 @@
 /* PTlink protocol module for IRC Services.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -81,7 +81,8 @@ static const struct modedata_init new_usermodes[] = {
 };
 
 static const struct modedata_init new_chanmodes[] = {
-    {'R', {0x00000100,0,0}},	/* Only identified users can join */
+    {'R', {0x00000100,0,0,0,MI_REGNICKS_ONLY}},
+				/* Only identified users can join */
     {'r', {0x00000200,0,0,0,MI_REGISTERED}},
 				/* Set for all registered channels */
     {'c', {0x00000400,0,0}},	/* No ANSI colors in channel */
@@ -515,9 +516,11 @@ static int do_check_chan_user_modes(const char *source, User *user,
 
 /*************************************************************************/
 
-static int do_check_kick(User *user, Channel *c, ChannelInfo *ci,
+static int do_check_kick(User *user, const char *chan, ChannelInfo *ci,
 			 char **mask_ret, const char **reason_ret)
 {
+    Channel *c = get_channel(chan);
+
     /* Don't let plain opers into +A (admin only) channels */
     if ((((c?c->mode:0) | (ci?ci->mlock_on:0)) & chanmode_admins_only)
      && !(user->mode & usermode_admin)
