@@ -1,6 +1,6 @@
 /* Unreal protocol module for IRC Services.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -22,6 +22,7 @@
 #include "banexcept.h"
 #include "chanprot.h"
 #include "halfop.h"
+#include "invitemask.h"
 #include "sjoin.h"
 #include "svsnick.h"
 #include "token.h"
@@ -183,7 +184,8 @@ static const struct modedata_init new_usermodes[] = {
  */
 
 static const struct modedata_init new_chanmodes[] = {
-    {'R', {0x00000100,0,0}},	/* Only identified users can join */
+    {'R', {0x00000100,0,0,0,MI_REGNICKS_ONLY}},
+				/* Only identified users can join */
     {'r', {0x00000200,0,0,0,MI_REGISTERED}},
 				/* Set for all registered channels */
     {'c', {0x00000400,0,0}},	/* No ANSI colors in channel */
@@ -207,8 +209,9 @@ static const struct modedata_init new_chanmodes[] = {
     {'L', {0x01000000,1,0}},	/* Channel link */
     {'M', {0x02000000,0,0}},	/* Moderated to unregged nicks */
     {'T', {0x04000000,0,0}},	/* Disable notices to channel */
-    {'e', {0x80000000,1,1,0,MI_MULTIPLE}},
-				/* Ban exceptions */
+    {'j', {0x08000000,1,0}},	/* Join throttling */
+    {'e', {0x80000000,1,1,0,MI_MULTIPLE}}, /* Ban exceptions */
+    {'I', {0x80000000,1,1,0,MI_MULTIPLE}}, /* INVITE hosts */
 };
 
 /*************************************************************************/
@@ -389,7 +392,7 @@ static void m_nick(char *source, int ac, char **av)
 
 static void m_protoctl(char *source, int ac, char **av)
 {
-    int got_nickv2 = 0;
+    static int got_nickv2 = 0;
     int i;
 
     for (i = 0; i < ac; i++) {
@@ -1061,7 +1064,7 @@ static int do_user_mode(User *user, int modechar, int add, char **av)
 /*************************************************************************/
 
 /*
- * Channel mode change callback; we handle modes L/f here.
+ * Channel mode change callback; we handle modes L/f/j here.
  */
 
 static int do_channel_mode(const char *source, Channel *channel,
@@ -1070,6 +1073,7 @@ static int do_channel_mode(const char *source, Channel *channel,
     int32 flag = mode_char_to_flag(modechar, MODE_CHANNEL);
 
     switch (modechar) {
+
       case 'L':
 	free(channel->link);
 	if (add) {
@@ -1091,7 +1095,46 @@ static int do_channel_mode(const char *source, Channel *channel,
 	    channel->flood = NULL;
 	}
 	return 1;
-    }
+
+      case 'j':
+        if (add) {
+	    int ok = 0;
+	    char *s;
+	    int joinrate1 = strtol(av[0], &s, 0);
+	    if (*s == ':') {
+		int joinrate2 = strtol(s+1, &s, 0);
+		if (!*s) {
+		    if (joinrate1 && joinrate2) {
+			channel->mode |= flag;
+			channel->joinrate1 = joinrate1;
+			channel->joinrate2 = joinrate2;
+		    } else {
+			channel->mode &= ~flag;
+			channel->joinrate1 = 0;
+			channel->joinrate2 = 0;
+			ok = 1;
+		    }
+		    ok = 1;
+		}
+	    } else if (joinrate1 == 0) {
+		channel->mode &= ~flag;
+		channel->joinrate1 = 0;
+		channel->joinrate2 = 0;
+		ok = 1;
+	    }
+	    if (!ok) {
+		module_log("warning: invalid MODE +j %s for %s", av[0],
+			   channel->name);
+	    }
+        } else {
+            channel->mode &= ~flag;
+            channel->joinrate1 = 0;
+            channel->joinrate2 = 0;
+        }
+        return 1;
+
+    } /* switch (mode) */
+
     return 0;
 }
 
@@ -1226,8 +1269,8 @@ static int do_set_topic(const char *source, Channel *c, const char *topic,
 /*************************************************************************/
 
 /*
- * ChanServ channel mode check callback; we handle setting modes +L and +f.
- * (-L and -f are handled by the default handler, which just sends a
+ * ChanServ channel mode check callback; we handle setting modes +L, +f,
+ * and +j. (-L/-f/-j are handled by the default handler, which just sends a
  * "MODE -L/-f" as appropriate.)
  */
 
@@ -1235,6 +1278,7 @@ static int do_check_modes(Channel *c, ChannelInfo *ci, int add, int32 flag)
 {
     if (add) {
 	switch (mode_flag_to_char(flag, MODE_CHANNEL)) {
+
 	  case 'L':
 	    if (!ci->mlock_link) {
 		module_log("warning: removing +L from channel %s mode lock"
@@ -1245,6 +1289,7 @@ static int do_check_modes(Channel *c, ChannelInfo *ci, int add, int32 flag)
 		    set_cmode(s_ChanServ, c, "+L", ci->mlock_link);
 	    }
 	    return 1;
+
 	  case 'f':
 	    if (!ci->mlock_flood) {
 		module_log("warning: removing +f from channel %s mode lock"
@@ -1255,8 +1300,31 @@ static int do_check_modes(Channel *c, ChannelInfo *ci, int add, int32 flag)
 		    set_cmode(s_ChanServ, c, "+f", ci->mlock_flood);
 	    }
 	    return 1;
-	}
-    }
+
+	  case 'j':
+	    if (sgn(ci->mlock_joinrate1) != sgn(ci->mlock_joinrate2)) {
+		module_log("warning: removing +j from channel %s mode lock"
+			   " (invalid parameter: %d:%d)", ci->name,
+			   ci->mlock_joinrate1, ci->mlock_joinrate2);
+		ci->mlock_on &= ~mode_char_to_flag('j', MODE_CHANNEL);
+		ci->mlock_joinrate1 = ci->mlock_joinrate2 = 0;
+	    } else if (ci->mlock_joinrate1 < 0) {
+		if (c->joinrate1 || c->joinrate2)
+		    set_cmode(s_ChanServ, c, "-j");
+	    } else {
+		if (c->joinrate1 != ci->mlock_joinrate1
+		 || c->joinrate2 != ci->mlock_joinrate2
+		) {
+		    char buf[BUFSIZE];
+		    snprintf(buf, sizeof(buf), "%d:%d",
+			     ci->mlock_joinrate1, ci->mlock_joinrate2);
+		    set_cmode(s_ChanServ, c, "+j", buf);
+		}
+	    }
+	    return 1;
+
+	} /* switch (modechar) */
+    } /* if (add) */
     return 0;
 }
 
@@ -1286,9 +1354,12 @@ static int do_check_chan_user_modes(const char *source, User *user,
  * secure-only, and no-hiding channel modes here.
  */
 
-static int do_check_kick(User *user, Channel *c, ChannelInfo *ci,
+static int do_check_kick(User *user, const char *chan, ChannelInfo *ci,
 			 char **mask_ret, const char **reason_ret)
 {
+    /* Retrieve the channel's Channel record, if present */
+    Channel *c = get_channel(chan);
+
     /* Don't do anything to service pseudoclients */
     if (user->mode & mode_char_to_flag('S', MODE_USER))
 	return 2;
@@ -1318,13 +1389,14 @@ static int do_check_kick(User *user, Channel *c, ChannelInfo *ci,
 	return 1;
     }
 
+    /* Let other processing continue as usual */
     return 0;
 }
 
 /*************************************************************************/
 
 /*
- * ChanServ SET MLOCK callback; we handle locking modes f, K, and L.
+ * ChanServ SET MLOCK callback; we handle locking modes f, j, K, and L.
  */
 
 static int do_set_mlock(User *u, ChannelInfo *ci, int mode, int add, char **av)
@@ -1347,8 +1419,11 @@ static int do_set_mlock(User *u, ChannelInfo *ci, int mode, int add, char **av)
     }
 
     /* Single mode set/clear */
+
     if (add) {
+
 	switch (mode) {
+
 	  case 'L':
 	    if (!valid_chan(av[0])) {
 		/* Invalid channel name */
@@ -1431,9 +1506,36 @@ static int do_set_mlock(User *u, ChannelInfo *ci, int mode, int add, char **av)
 	    /* String is invalid */
 	    notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_BAD_PARAM, mode);
 	    return 1;
-	  }
-	}
-    }
+	  } /* case 'f' */
+
+          case 'j': {
+	    int ok = 0;
+	    char *s;
+	    ci->mlock_joinrate1 = strtol(av[0], &s, 0);
+	    if (ci->mlock_joinrate1 > 0 && *s == ':') {
+		ci->mlock_joinrate2 = strtol(s+1, &s, 0);
+		if (ci->mlock_joinrate2 > 0 && !*s)
+		    ok = 1;
+	    }
+	    if (!ok) {
+		notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_BAD_PARAM, mode);
+		return 1;
+	    }
+	    break;
+	  } /* case 'j' */
+
+	} /* switch (mode) */
+
+    } else {  /* !add -> lock off */
+
+        switch (mode) {
+	  case 'j':
+	    ci->mlock_joinrate1 = ci->mlock_joinrate2 = -1;
+	    break;
+	} /* switch (mode) */
+
+    } /* if (add) */
+
     return 0;
 }
 
@@ -1721,6 +1823,7 @@ int init_module(Module *module_)
     if (!init_banexcept(module)
      || !init_chanprot(module)
      || !init_halfop(module)
+     || !init_invitemask(module)
      || !init_sjoin(module)
      || !init_svsnick(module)
      || !init_token(module, tokens)
@@ -1794,6 +1897,7 @@ int exit_module(int shutdown)
     exit_token();
     exit_svsnick();
     exit_sjoin();
+    exit_invitemask();
     exit_halfop();
     exit_chanprot();
     exit_banexcept();
