@@ -2,7 +2,7 @@
  * Based on code (c) Andrew Kempe (TheShadow)
  *     E-mail: <andrewk@isdial.net>
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -656,7 +656,7 @@ int exit_module(int shutdown_unused)
     if (module_nickserv)
 	do_unload_module(module_nickserv);
 
-    remove_callback(NULL, "save data", sync_statserv_db);
+    remove_callback(NULL, "save data", do_save_data);
     remove_callback(NULL, "user MODE", stats_do_umode);
     remove_callback(NULL, "user delete", stats_do_quit);
     remove_callback(NULL, "user create", stats_do_newuser);
