@@ -1,6 +1,6 @@
 /* Socket routines.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -189,9 +189,7 @@ void sock_setcb(Socket *s, int which, SocketCallback func)
 
 /*************************************************************************/
 
-/* Return whether the given socket is currently connected (this includes
- * sockets which are waiting for a connection to be completed).
- */
+/* Return whether the given socket is currently connected. */
 
 int sock_isconn(const Socket *s)
 {
@@ -200,7 +198,7 @@ int sock_isconn(const Socket *s)
 	errno = EINVAL;
 	return 0;
     }
-    return s->flags & (SF_CONNECTING|SF_CONNECTED) ? 1 : 0;
+    return s->flags & SF_CONNECTED ? 1 : 0;
 }
 
 /*************************************************************************/
@@ -579,7 +577,7 @@ int conn(Socket *s, const char *host, int port, const char *lhost, int lport)
 #if HAVE_GETHOSTBYNAME
     struct hostent *hp;
 #endif
-    char *addr;
+    uint8 *addr;
     struct sockaddr_in sa, lsa;
     int fd, i;
 
@@ -722,7 +720,7 @@ int open_listener(Socket *s, const char *host, int port, int backlog)
 #if HAVE_GETHOSTBYNAME
     struct hostent *hp;
 #endif
-    char *addr;
+    uint8 *addr;
     struct sockaddr_in sa;
     int fd, i;
 
@@ -1162,6 +1160,8 @@ static int flush_write_buffer(Socket *s)
 	errno = EBADF;
 	return -1;
     }
+    if (!sock_isconn(s))  /* not yet connected */
+	return 0;
     if (s->wend != s->wptr) {
 	if (s->wptr > s->wend)	/* wrapped around? */
 	    maxwrite = s->wtop - s->wptr;
