@@ -1,6 +1,6 @@
 /* Convert other programs' databases to XML.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -173,7 +173,7 @@ dbFILE *open_db_ver(const char *dir, const char *name, int32 min_version,
     snprintf(filename, sizeof(filename), "%s/%s", dir, name);
     f = open_db(filename, "r", 0);
     if (!f) {
-	fprintf(stderr, "Can't open %s for reading: %s", filename,
+	fprintf(stderr, "Can't open %s for reading: %s\n", filename,
 		strerror(errno));
 	exit(1);
     }
@@ -406,26 +406,54 @@ int get_operserv_data(int what, void *ptr)
 
 /* Perform sanity checks on the data after it's been read in. */
 
+static void sanity_check_nicks(void);
+static void sanity_check_nickgroups(void);
+static void sanity_check_channels(void);
+static void sanity_check_maskdata(void);
+
 static void sanity_checks(void)
+{
+    sanity_check_nicks();
+    sanity_check_nickgroups();
+    sanity_check_channels();
+    sanity_check_maskdata();
+}
+
+static void sanity_check_nicks(void)
 {
     NickInfo *ni;
     NickGroupInfo *ngi;
-    ChannelInfo *ci;
+    char *s;
     int i;
 
-    /* Check for non-forbidden nicknames with no nickgroup */
     for (ni = first_nickinfo(); ni; ni = next_nickinfo()) {
-	if (!(ni->status & NS_VERBOTEN) && !ni->nickgroup) {
+
+	/* Forbidden nicks should have no other data associated with them. */
+	if (ni->status & NS_VERBOTEN) {
+	    ni->status &= ~(NS_VERBOTEN);
+	    ni->last_usermask = NULL;
+	    ni->last_realmask = NULL;
+	    ni->last_realname = NULL;
+	    ni->last_quit = NULL;
+	    ni->last_seen = 0;
+	    if (ni->nickgroup) {
+		fprintf(stderr, "BUG: Forbidden nickname %s associated with"
+			" nickgroup %u!  Clearing nickgroup field.\n",
+			ni->nick, ni->nickgroup);
+		ni->nickgroup = 0;
+	    }
+	    ni->id_stamp = 0;
+	    continue;
+	}
+
+	/* The nick isn't a forbidden nick.  First make sure it has the
+	 * right nickgroup. */
+	if (!ni->nickgroup) {
 	    fprintf(stderr, "BUG: Nickname %s has no nickgroup!  Deleting.\n",
 		    ni->nick);
 	    del_nickinfo(ni);
-	}
-    }
-
-    /* Make sure every non-forbidden nick belongs to the right nickgroup */
-    for (ni = first_nickinfo(); ni; ni = next_nickinfo()) {
-	if (ni->status & NS_VERBOTEN)
 	    continue;
+	}
 	ngi = get_nickgroupinfo(ni->nickgroup);
 	if (!ngi) {
 	    fprintf(stderr, "BUG: Nickname %s points to nonexistent nickgroup"
@@ -443,10 +471,52 @@ static void sanity_checks(void)
 		del_nickinfo(ni);
 	    }
 	}
-    }
 
-    /* Make sure nickgroups don't contain extra nicks. */
+	/* Clear unknown flags. */
+	ni->status &= NS_PERMANENT;
+
+	/* Make sure usermasks actually have non-empty user and host parts. */
+	if (ni->last_usermask) {
+	    s = strchr(ni->last_usermask, '@');
+	    if (!s || s==ni->last_usermask || !s[1]) {
+		fprintf(stderr, "Last usermask for nickname %s isn't a valid"
+			" user@host mask, clearing.\n", ni->nick);
+		ni->last_usermask = NULL;
+	    }
+	}
+	if (ni->last_realmask) {
+	    s = strchr(ni->last_realmask, '@');
+	    if (!s || s==ni->last_realmask || !s[1]) {
+		fprintf(stderr, "Last real usermask for nickname %s isn't"
+			" a valid user@host mask, clearing.\n", ni->nick);
+		ni->last_realmask = NULL;
+	    }
+	}
+
+	/* Make sure last-seen time is not earlier than registered time.
+	 * Allow zero, to accommodate cases where the nickname was
+	 * registered by an outside entity without the user actually
+	 * logging on. */
+	if (ni->last_seen && ni->last_seen < ni->time_registered) {
+	    fprintf(stderr, "Last-seen time for nickname %s is earlier than"
+		    " registration time!  Setting last-seen time to"
+		    " registration time.\n", ni->nick);
+	    ni->last_seen = ni->time_registered;
+	}
+
+    }  /* for all nicks */
+}
+
+static void sanity_check_nickgroups(void)
+{
+    NickInfo *ni;
+    NickGroupInfo *ngi;
+    char *s;
+    int i;
+
     for (ngi = first_nickgroupinfo(); ngi; ngi = next_nickgroupinfo()) {
+
+	/* Make sure nickgroups don't contain extra nicks. */
 	ARRAY_FOREACH(i, ngi->nicks) {
 	    ni = get_nickinfo(ngi->nicks[i]);
 	    if (!ni) {
@@ -454,50 +524,278 @@ static void sanity_checks(void)
 			" nickname %s!  Removing nickname from group.\n",
 			ngi->id, ngi->nicks[i]);
 		ARRAY_REMOVE(ngi->nicks, i);
+		if (ngi->mainnick >= i)
+		    ngi->mainnick--;
 		i--;  /* to make sure we don't skip any */
-		if (ngi->nicks_count <= 0)
-		    del_nickgroupinfo(ngi);
 	    }
 	}
-    }
 
-    /* Check for non-forbidden channels with no founder */
+	/* Check for empty nickgroups (possibly from extraneous nickname
+	 * removal above). */
+	if (ngi->nicks_count == 0) {
+	    fprintf(stderr, "Removing empty nickgroup %u.\n", ngi->id);
+	    del_nickgroupinfo(ngi);
+	    continue;
+	}
+
+	/* Make sure main nick is a valid index. */
+	if (ngi->mainnick >= ngi->nicks_count) {
+	    fprintf(stderr, "Invalid main nick index in nickgroup %u,"
+		    " resetting.\n", ngi->id);
+	    ngi->mainnick = 0;
+	}
+
+	/* If an authcode is set, make sure the reason is valid.  If not,
+	 * clear the auth-related fields. */
+	if (ngi->authcode && (ngi->authreason < NICKAUTH_MIN
+			   || ngi->authreason > NICKAUTH_MAX)) {
+	    fprintf(stderr, "Authentication code set for nickgroup %u but"
+		    " reason code invalid, setting to SETAUTH.\n", ngi->id);
+	    ngi->authreason = NICKAUTH_SETAUTH;
+	} else {
+	    ngi->authset = 0;
+	    ngi->authreason = 0;
+	}
+
+	/* Clear all unknown flags. */
+	ngi->flags &= NF_ALLFLAGS;
+
+	/* Make sure language setting is in range. */
+	if ((ngi->language < 0 || ngi->language >= NUM_LANGS)
+	 && ngi->language != LANG_DEFAULT
+	) {
+	    fprintf(stderr, "Invalid language set for nickgroup %u,"
+		    " resetting to default.\n", ngi->id);
+	    ngi->language = LANG_DEFAULT;
+	}
+
+	/* Make sure access, autojoin, and ignore counts are non-negative. */
+	if (ngi->access_count < 0) {
+	    fprintf(stderr, "BUG: Access entry count for nickgroup %u is"
+		    " negative!  Clearing.\n", ngi->id);
+	    ngi->access = NULL;
+	    ngi->access_count = 0;
+	}
+	if (ngi->ajoin_count < 0) {
+	    fprintf(stderr, "BUG: Autojoin entry count for nickgroup %u is"
+		    " negative!  Clearing.\n", ngi->id);
+	    ngi->ajoin = NULL;
+	    ngi->ajoin_count = 0;
+	}
+	if (ngi->ignore_count < 0) {
+	    fprintf(stderr, "BUG: Ignore entry count for nickgroup %u is"
+		    " negative!  Clearing.\n", ngi->id);
+	    ngi->ignore = NULL;
+	    ngi->ignore_count = 0;
+	}
+
+	/* Make sure all access entries have non-empty user and host parts. */
+	ARRAY_FOREACH (i, ngi->access) {
+	    if (!ngi->access[i]
+	     || !(s = strchr(ngi->access[i], '@'))
+	     || s == ngi->access[i]
+	     || !s[1]
+	    ) {
+		fprintf(stderr, "Access entry %d for nickgroup %u %s,"
+			" deleting.\n", i, ngi->id,
+			!ngi->access[i] ? "is empty"
+			                : "isn't a valid user@host mask");
+		ARRAY_REMOVE(ngi->access, i);
+		i--;
+	    }
+	}
+
+	/* Make sure memo count is non-negative. */
+	if (ngi->memos.memos_count < 0) {
+	    fprintf(stderr, "BUG: Memo count for nickgroup %u is negative!"
+		    "  Clearing.\n", ngi->id);
+	    ngi->memos.memos = NULL;
+	    ngi->memos.memos_count = 0;
+	}
+
+	/* Clear unknown flags from memos. */
+	ARRAY_FOREACH (i, ngi->memos.memos)
+	    ngi->memos.memos[i].flags &= MF_ALLFLAGS;
+
+    }  /* for all nickgroups */
+}
+
+static void sanity_check_channels(void)
+{
+    ChannelInfo *ci;
+    char *s;
+    int i;
+
     for (ci = first_channelinfo(); ci; ci = next_channelinfo()) {
-	if (!(ci->flags & CI_VERBOTEN) && !ci->founder) {
+
+	/* Forbidden channels should have no other data associated with
+	 * them. */
+	if (ci->flags & CI_VERBOTEN) {
+	    ci->founder = 0;
+	    ci->successor = 0;
+	    memset(ci->founderpass, 0, sizeof(ci->founderpass));
+	    ci->desc = NULL;
+	    ci->url = NULL;
+	    ci->email = NULL;
+	    ci->last_used = 0;
+	    ci->last_topic = NULL;
+	    memset(ci->last_topic_setter, 0, sizeof(ci->last_topic_setter));
+	    ci->last_topic_time = 0;
+	    ci->flags &= ~(CI_VERBOTEN);
+	    ci->suspendinfo = NULL;
+	    ci->levels = NULL;
+	    ci->access = NULL;
+	    ci->access_count = 0;
+	    ci->akick = NULL;
+	    ci->akick_count = 0;
+	    ci->mlock_on = NULL;
+	    ci->mlock_off = NULL;
+	    ci->mlock_limit = 0;
+	    ci->mlock_key = NULL;
+	    ci->mlock_link = NULL;
+	    ci->mlock_flood = NULL;
+	    ci->mlock_joindelay = 0;
+	    ci->mlock_joinrate1 = 0;
+	    ci->mlock_joinrate2 = 0;
+	    ci->entry_message = NULL;
+	    ci->memos.memos = NULL;
+	    ci->memos.memos_count = 0;
+	    ci->memos.memomax = MEMOMAX_DEFAULT;
+	    continue;
+	}
+
+	/* Channel is not forbidden.  First make sure it has a (valid)
+	 * founder. */
+	if (!ci->founder) {
 	    fprintf(stderr, "BUG: Channel %s has no founder!  Deleting.\n",
 		    ci->name);
 	    del_channelinfo(ci);
+	} else if (!get_nickgroupinfo(ci->founder)) {
+	    fprintf(stderr, "BUG: Channel %s founder nickgroup %u is"
+		    " missing!  Deleting channel.\n", ci->name, ci->founder);
+	    del_channelinfo(ci);
 	}
-    }
+
+	/* Make sure that the successor is valid if set. */
+	if (ci->successor && !get_nickgroupinfo(ci->successor)) {
+	    fprintf(stderr, "BUG: Channel %s successor nickgroup %u is"
+		    " missing!  Clearing.", ci->name, ci->successor);
+	    ci->successor = 0;
+	}
+
+	/* Make sure last-used time is not earlier than registered time.
+	 * Allow zero, to accommodate cases where the channel was
+	 * registered by an outside entity. */
+	if (ci->last_used && ci->last_used < ci->time_registered) {
+	    fprintf(stderr, "Last-used time for channel %s is earlier than"
+		    " registration time!  Setting last-used time to"
+		    " registration time.\n", ci->name);
+	    ci->last_used = ci->time_registered;
+	}
+
+	/* If there is no saved topic, clear the topic setter and time. */
+	if (!ci->last_topic) {
+	    memset(ci->last_topic_setter, 0, sizeof(ci->last_topic_setter));
+	    ci->last_topic_time = 0;
+	}
+
+	/* Clear unknown flags. */
+	ci->flags &= CI_ALLFLAGS;
+
+	/* Check privilege level settings for vaility. */
+	if (ci->levels) {
+	    for (i = 0; i < CA_SIZE; i++) {
+		if (ci->levels[i] < ACCLEV_INVALID
+		 || ci->levels[i] > ACCLEV_FOUNDER
+		) {
+		    fprintf(stderr, "Privilege level %d on channel %s is"
+			    "invalid; resetting all levels to defaults.\n",
+			    i, ci->name);
+		    ci->levels = NULL;
+		    break;
+		}
+	    }
+	}
+
+	/* Check access level nickgroups and levels for validity. */
+	ARRAY_FOREACH (i, ci->access) {
+	    if (!ci->access[i].nickgroup)
+		continue;
+	    if (!get_nickgroupinfo(ci->access[i].nickgroup)) {
+		fprintf(stderr, "BUG: Channel %s access entry %d has an"
+			" invalid nickgroup!  Clearing.\n", ci->name, i);
+		ci->access[i].nickgroup = 0;
+		ci->access[i].level = 0;
+	    } else if (ci->access[i].level <= ACCLEV_INVALID
+		    || ci->access[i].level >= ACCLEV_FOUNDER) {
+		fprintf(stderr, "BUG: Channel %s access entry %d has an"
+			"out-of-range level (%d)!  Clearing.\n",
+			ci->name, i, ci->access[i].level);
+		ci->access[i].nickgroup = 0;
+		ci->access[i].level = 0;
+	    }
+	}
+
+	/* Make sure all autokick entries have non-empty user and host
+	 * parts. */
+	ARRAY_FOREACH (i, ci->akick) {
+	    if (!ci->akick[i].mask)
+		continue;
+	    s = strchr(ci->akick[i].mask, '@');
+	    if (!s || s==ci->akick[i].mask || !s[1]) {
+		fprintf(stderr, "Autokick entry %d for channel %s isn't a"
+			" valid user@host mask, deleting.\n", i, ci->name);
+		ci->akick[i].mask = NULL;
+	    }
+	}
+
+	/* Make sure memo count is non-negative. */
+	if (ci->memos.memos_count < 0) {
+	    fprintf(stderr, "BUG: Memo count for channel %s is negative!"
+		    "  Clearing.\n", ci->name);
+	    ci->memos.memos = NULL;
+	    ci->memos.memos_count = 0;
+	}
+
+	/* Clear unknown flags from memos. */
+	ARRAY_FOREACH (i, ci->memos.memos)
+	    ci->memos.memos[i].flags &= MF_ALLFLAGS;
+
+    }  /* for all channels */
 
     /* Check that channel successors and access list entries have valid
      * nickgroup IDs, and that access levels are in range */
     for (ci = first_channelinfo(); ci; ci = next_channelinfo()) {
 	if (ci->flags & CI_VERBOTEN)
 	    continue;
-	if (ci->successor && !get_nickgroupinfo(ci->successor)) {
-	    fprintf(stderr, "BUG: Channel %s successor is an invalid"
-		    " nickgroup!  Clearing.\n", ci->name);
-	    ci->successor = 0;
+    }
+}
+
+static void sanity_check_maskdata(void)
+{
+    MaskData *md;
+    char *s;
+    int i;
+
+    /* Make sure all MaskData entries actually have masks allocated. */
+    for (i = 0; i < 256; i++) {
+	for (md = first_maskdata(i); md; md = next_maskdata(i)) {
+	    if (!md->mask)
+		del_maskdata(i, md);
 	}
-	ARRAY_FOREACH (i, ci->access) {
-	    if (ci->access[i].nickgroup) {
-		if (!get_nickgroupinfo(ci->access[i].nickgroup)) {
-		    fprintf(stderr, "BUG: Channel %s access entry %d has"
-			    " an invalid nickgroup!  Clearing.\n",
-			    ci->name, i);
-		    ci->access[i].nickgroup = 0;
-		    ci->access[i].level = 0;
-		} else if (ci->access[i].level <= ACCLEV_INVALID
-			   || ci->access[i].level >= ACCLEV_FOUNDER) {
-		    fprintf(stderr, "BUG: Channel %s access entry %d has"
-			    " an out-of-range level (%d)!  Clearing.\n",
-			    ci->name, i, ci->access[i].level);
-		    ci->access[i].nickgroup = 0;
-		    ci->access[i].level = 0;
-		}
-	    }
+    }
+
+    /* Make sure every autokill has a valid user@host mask and a reason. */
+    for (md = first_maskdata(MD_AKILL); md; md = next_maskdata(MD_AKILL)) {
+	s = strchr(md->mask, '@');
+	if (!s || s==md->mask || !s[1]) {
+	    fprintf(stderr, "Autokill %s isn't a valid user@host mask,"
+		    " deleting.\n", md->mask);
+	    del_maskdata(MD_AKILL, md);
+	    continue;
 	}
+	if (!md->reason)
+	    md->reason = "Reason unknown";
     }
 }
 
