@@ -1,6 +1,6 @@
 /* Conversion routines for PTlink >= 2.13.x databases.
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -29,7 +29,7 @@ static void ptlink_load_nick(const char *sourcedir)
 {
     char filename[PATH_MAX+1];
     dbFILE *f;
-    int i, j, c;
+    int i, j, c, delete;
     int16 tmp16, ver;
     int32 tmp32, total, count;
     NickInfo *ni;
@@ -42,7 +42,7 @@ static void ptlink_load_nick(const char *sourcedir)
 	return;
     SAFE(read_int8(&ch, f));
     SAFE(read_int16(&ver, f));
-    if (ch != -1 || ver < 6 || ver > 10) {
+    if (ch != -1 || ver < 6 || ver > 11) {
 	fprintf(stderr, "Wrong version number on %s\n", filename);
 	exit(1);
     }
@@ -56,7 +56,7 @@ static void ptlink_load_nick(const char *sourcedir)
 	while (ver >= 10 || (c = getc_db(f)) == 1) {
 	    char nickbuf[32], passbuf[32], authbuf[16];
 	    if (ver >= 9) {
-		int res = read_int32(&tmp32, f);  /* suid */
+		int res = read_int32(&tmp32, f);  /* SUID (broken) / SNID */
 		if (res < 0) {
 		    if (ver >= 10)  /* v10: no more int8 markers */
 			break;
@@ -65,13 +65,23 @@ static void ptlink_load_nick(const char *sourcedir)
 		}
 	    }
 	    SAFE(read_buffer(nickbuf, f));
+	    /* The source (v2.26-eol.1) claims that duplicate nicknames can
+	     * get stored in the database for unknown reasons, so watch for
+	     * and delete them as needed */
+	    delete = (get_nickinfo(nickbuf) != NULL);
 	    ni = makenick(nickbuf, &ngi);
+	    /* Use the SNID (file version 11) as a nickname group ID only
+	     * if it's nonzero and unique.  Never use the SUID (versions 9
+	     * and 10) as the source code claims that it's broken. */
+	    if (ver >= 11 && tmp32 && !get_nickgroupinfo(tmp32))
+		ni->nickgroup = ngi->id = (uint32)tmp32;
 	    SAFE(read_buffer(passbuf, f));
 	    memcpy(ngi->pass, passbuf, sizeof(passbuf));
 	    if (ver >= 7)
 		SAFE(read_buffer(authbuf, f));
 	    SAFE(read_string(&ngi->url, f));
-	    SAFE(read_string(&s, f));  /* temporary, unauthed E-mail */
+	    if (ver >= 7)
+		SAFE(read_string(&s, f));  /* temporary, unauthed E-mail */
 	    SAFE(read_string(&ngi->email, f));
 	    SAFE(read_string(&s, f));  /* icq_number */
 	    SAFE(read_string(&s, f));  /* location */
@@ -87,7 +97,8 @@ static void ptlink_load_nick(const char *sourcedir)
 	    SAFE(read_int32(&tmp32, f));  /* last_identify */
 	    SAFE(read_int32(&tmp32, f));
 	    ni->last_seen = tmp32;
-	    SAFE(read_int32(&tmp32, f));  /* last_email_request */
+	    if (ver >= 7)
+		SAFE(read_int32(&tmp32, f));  /* last_email_request */
 	    SAFE(read_int32(&tmp32, f));  /* birth_date */
 	    SAFE(read_int16(&tmp16, f));  /* status */
 	    if (tmp16 & 0x0002) {
@@ -174,7 +185,7 @@ static void ptlink_load_nick(const char *sourcedir)
 		    memos = scalloc(sizeof(*memos), ngi->memos.memos_count);
 		    ngi->memos.memos = memos;
 		    for (j = 0; j < ngi->memos.memos_count; j++, memos++) {
-			SAFE(read_int32(&memos->number, f));
+			SAFE(read_uint32(&memos->number, f));
 			SAFE(read_int16(&tmp16, f));
 			if (tmp16 & 1)
 			    memos->flags |= MF_UNREAD;
@@ -201,6 +212,10 @@ static void ptlink_load_nick(const char *sourcedir)
 		    case 4:  ngi->language = LANG_IT;      break;
 		}
 	    }
+	    if (delete) {
+		del_nickgroupinfo(ngi);
+		del_nickinfo(ni);
+	    }
 	    count++;
 	} /* while (getc_db(f) == 1) */
 	if (ver < 10 && c != 0) {
@@ -287,7 +302,7 @@ static void ptlink_load_chan(const char *sourcedir)
 {
     char filename[PATH_MAX+1];
     dbFILE *f;
-    int i, j, c;
+    int i, j, c, C_MAX;
     ChannelInfo *ci;
     NickInfo *ni;
     signed char ch;
@@ -300,7 +315,7 @@ static void ptlink_load_chan(const char *sourcedir)
 	return;
     SAFE(read_int8(&ch, f));
     SAFE(read_int16(&ver, f));
-    if (ch != -1 || ver < 7 || ver > 9) {
+    if (ch != -1 || ver < 7 || ver > 10) {
 	fprintf(stderr, "Wrong version number on %s\n", filename);
 	exit(1);
     }
@@ -309,11 +324,14 @@ static void ptlink_load_chan(const char *sourcedir)
 	SAFE(read_int32(&total, f));
     count = 0;
 
-    for (i = 0; i < 65535; i++) {
+    C_MAX = (ver<9 ? 256 : 65535);
+    for (i = 0; i < C_MAX; i++) {
 	char *s;
 
 	while ((c = getc_db(f)) == 1) {
 	    char namebuf[64], passbuf[32], nickbuf[32];
+	    if (ver >= 10)
+		SAFE(read_int32(&tmp32, f));  /* SCID */
 	    SAFE(read_buffer(namebuf, f));
 	    ci = makechan(namebuf);
 	    SAFE(read_string(&s, f));
@@ -353,7 +371,7 @@ static void ptlink_load_chan(const char *sourcedir)
 	    memcpy(ci->founderpass, passbuf, sizeof(passbuf));
 	    SAFE(read_string(&ci->desc, f));
 	    if (!ci->desc)
-		(char *)ci->desc = (char *)"";
+		ci->desc = (char *)"";
 	    SAFE(read_string(&ci->url, f));
 	    SAFE(read_string(&ci->email, f));
 	    SAFE(read_int32(&tmp32, f));
@@ -475,7 +493,7 @@ static void ptlink_load_chan(const char *sourcedir)
 		memos = scalloc(sizeof(Memo), ci->memos.memos_count);
 		ci->memos.memos = memos;
 		for (j = 0; j < ci->memos.memos_count; j++, memos++) {
-		    SAFE(read_int32(&memos->number, f));
+		    SAFE(read_uint32(&memos->number, f));
 		    SAFE(read_int16(&tmp16, f));
 		    if (tmp16 & 1)
 			memos->flags |= MF_UNREAD;
