@@ -1,7 +1,7 @@
 /* Conversion routines for datafile version 8 based databases (Daylight and
  * IRCS 1.2).
  *
- * IRC Services is copyright (c) 1996-2005 Andrew Church.
+ * IRC Services is copyright (c) 1996-2007 Andrew Church.
  *     E-mail: <achurch@achurch.org>
  * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
@@ -123,7 +123,7 @@ static void ver8_load_nick(const char *sourcedir, int type)
 		    memos = scalloc(sizeof(Memo), ngi->memos.memos_count);
 		    ngi->memos.memos = memos;
 		    for (j = 0; j < ngi->memos.memos_count; j++, memos++) {
-			SAFE(read_int32(&memos->number, f));
+			SAFE(read_uint32(&memos->number, f));
 			SAFE(read_int16(&tmp16, f));
 			if (tmp16 & 1)
 			    memos->flags |= MF_UNREAD;
@@ -423,7 +423,7 @@ static void ver8_load_chan(const char *sourcedir, int type)
 		memos = scalloc(sizeof(Memo), ci->memos.memos_count);
 		ci->memos.memos = memos;
 		for (j = 0; j < ci->memos.memos_count; j++, memos++) {
-		    SAFE(read_int32(&memos->number, f));
+		    SAFE(read_uint32(&memos->number, f));
 		    SAFE(read_int16(&memos->flags, f));
 		    SAFE(read_int32(&tmp32, f));
 		    memos->time = tmp32;
